@@ -85,18 +85,18 @@ gda_row_class_init (GdaRowClass *klass)
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GdaRowClass, value_changed),
 			      NULL, NULL,
-			      gda_marshal_VOID__INT_POINTER_POINTER,
+			      gda_marshal_VOID__INT_BOXED_BOXED,
 			      G_TYPE_NONE,
-			      3, G_TYPE_INT, G_TYPE_POINTER, G_TYPE_POINTER);
+			      3, G_TYPE_INT, G_TYPE_VALUE, G_TYPE_VALUE);
 	gda_row_signals[VALUE_TO_CHANGE] =
 		g_signal_new ("value_to_change",
 			      G_TYPE_FROM_CLASS (object_class),
 			      G_SIGNAL_RUN_LAST,
 			      G_STRUCT_OFFSET (GdaRowClass, value_to_change),
 			      NULL, NULL,
-			      gda_marshal_VOID__INT_POINTER_POINTER,
+			      gda_marshal_VOID__INT_BOXED_BOXED,
 			      G_TYPE_BOOLEAN,
-			      3, G_TYPE_INT, G_TYPE_POINTER, G_TYPE_POINTER);
+			      3, G_TYPE_INT, G_TYPE_VALUE, G_TYPE_VALUE);
 
 	object_class->finalize = gda_row_finalize;
 	object_class->dispose = gda_row_dispose;
@@ -346,7 +346,7 @@ gda_row_copy (GdaRow *row)
  * @values: a list of #GValue's.
  *
  * Creates a #GdaRow from a list of #GValue's.  These GValue's are
- * value-copied and the user are still responsible for freeing them.
+ * value-copied and the user is still responsible for freeing them with g_object_unref().
  *
  * See the gda_row_new() function's documentation for more information about the @model attribute
  *
@@ -369,10 +369,9 @@ gda_row_new_from_list (GdaDataModel *model, const GList *values)
  * @row: a #GdaRow.
  * @model: a #GdaDataModel this row belongs to, or %NULL if the row is outside any data model
  *
- * Set the #GdaDataModel the given #GdaRow belongs to. Note that calling this method should be reserved
- * to GdaDataModel implementations and should therefore not be called by the user.
- *
- * Returns: a #GdaDataModel.
+ * Set the #GdaDataModel the given #GdaRow belongs to. Note that calling
+ * this method should be reserved to GdaDataModel implementations and should
+ * therefore not be called by the user.
  */
 void
 gda_row_set_model (GdaRow *row, GdaDataModel *model)
@@ -492,7 +491,7 @@ gda_row_set_id (GdaRow *row, const gchar *id)
  * Gets a pointer to a #GValue stored in a #GdaRow.
  *
  * This is a pointer to the internal array of values. Don't try to free
- * or modify it!
+ * or modify it.
  *
  * Returns: a pointer to the #GValue in the position @num of @row.
  */
