@@ -749,17 +749,25 @@ load_xml_spec (GdaServerOperation *op, xmlNodePtr specnode, const gchar *root)
 
 		/* don't care about entries for the wrong locale */
 		this_lang = xmlGetProp(node, (xmlChar*)"lang");
-		if (this_lang && strncmp ((gchar*)this_lang, lang, strlen ((gchar*)this_lang))) {
-			g_free (this_lang);
+		if (this_lang) {
+			if (strncmp ((gchar*)this_lang, lang, strlen ((gchar*)this_lang))) {
+				xmlFree (this_lang);
+				node = node->next;
+				continue;
+			}
+
+			xmlFree(this_lang);
+		}
+
+		id = xmlGetProp (node, BAD_CAST "id");
+		if (!id) {
 			node = node->next;
 			continue;
 		}
 
-		id = xmlGetProp (node, BAD_CAST "id");
 		complete_path = g_strdup_printf ("%s/%s", root ? root : "", id);
 		path_name = g_strdup ((gchar*)id);
-		if (id)
-			xmlFree (id);
+		xmlFree (id);
 
 		old_opnode = node_find (op, complete_path);
 		if (old_opnode) {
@@ -1028,6 +1036,7 @@ gda_server_operation_get_node_info (GdaServerOperation *op, const gchar *path_fo
 					g_object_get (G_OBJECT (column), "id", &colid, NULL);
 					if (!colid || strcmp (colid, extension +1))
 						column = NULL;
+					g_free(colid);
 				}
 			}
 			g_free (extension);
@@ -1821,6 +1830,7 @@ gda_server_operation_get_value_at (GdaServerOperation *op, const gchar *path_for
 					value = gda_data_model_get_value_at (node_info->model, 
 									     gda_column_get_position (node_info->column), 
 									     row);
+				g_free(extension);
 			}
 			g_free (str);
 		}		
@@ -1950,6 +1960,7 @@ gda_server_operation_set_value_at (GdaServerOperation *op, const gchar *value, G
 					g_object_get (G_OBJECT (column), "id", &colid, NULL);
 					if (!colid || strcmp (colid, colname +1))
 						column = NULL;
+					g_free(colid);
 				}
 				if (column) {
 					gchar *ptr;
@@ -2000,13 +2011,14 @@ gda_server_operation_set_value_at (GdaServerOperation *op, const gchar *value, G
 
 	g_free (extension);
 	g_free (colname);
+	g_free (path);
 	return allok;
 }
 
 /**
  * gda_server_operation_is_valid
  * @op: a #GdaServerOperation widget
- * @xml_file: a XML specification file (see gda_server_operation_new())
+ * @xml_file: an XML specification file (see gda_server_operation_new())
  * @error: a place to store an error, or %NULL
  *
  * Tells if all the required values in @op have been defined.
