@@ -1,5 +1,5 @@
 /* GDA library
- * Copyright (C) 1998 - 2006 The GNOME Foundation.
+ * Copyright (C) 1998 - 2007 The GNOME Foundation.
  *
  * AUTHORS:
  *	Rodrigo Moya <rodrigo@gnome-db.org>
@@ -28,6 +28,8 @@
 #include <libgda/gda-server-provider-private.h>
 #include <libgda/gda-data-handler.h>
 #include <libgda/gda-parameter-list.h>
+#include <libgda/gda-renderer.h>
+#include <libgda/gda-util.h>
 #include <string.h>
 #include <glib/gi18n-lib.h>
 
@@ -45,20 +47,6 @@ static GObjectClass *parent_class = NULL;
  * GdaServerProvider class implementation
  */
 
-static GdaBlob *
-_gda_server_provider_create_blob (GdaServerProvider *provider,
-				  GdaConnection *cnc)
-{
-	return NULL;
-}
-
-static GdaBlob *
-_gda_server_provider_fetch_blob (GdaServerProvider *provider,
-				 GdaConnection *cnc, const gchar *sql_id)
-{
-	return NULL;
-}
-
 static void
 gda_server_provider_class_init (GdaServerProviderClass *klass)
 {
@@ -92,8 +80,6 @@ gda_server_provider_class_init (GdaServerProviderClass *klass)
 	klass->begin_transaction = NULL;
 	klass->commit_transaction = NULL;
 	klass->rollback_transaction = NULL;
-	klass->create_blob = _gda_server_provider_create_blob;
-	klass->fetch_blob = _gda_server_provider_fetch_blob;
 }
 
 static void
@@ -103,7 +89,6 @@ gda_server_provider_init (GdaServerProvider *provider,
 	g_return_if_fail (GDA_IS_SERVER_PROVIDER (provider));
 
 	provider->priv = g_new0 (GdaServerProviderPrivate, 1);
-	provider->priv->connections = NULL;
 	provider->priv->data_handlers = g_hash_table_new_full ((GHashFunc) gda_server_provider_handler_info_hash_func,
 							       (GEqualFunc) gda_server_provider_handler_info_equal_func,
 							       (GDestroyNotify) gda_server_provider_handler_info_free,
@@ -119,7 +104,6 @@ gda_server_provider_finalize (GObject *object)
 
 	/* free memory */
 	if (provider->priv) {
-		g_list_free (provider->priv->connections);
 		g_hash_table_destroy (provider->priv->data_handlers);
 		
 		g_free (provider->priv);
@@ -247,15 +231,6 @@ gda_server_provider_open_connection (GdaServerProvider *provider,
 	}
 
 	retcode = CLASS (provider)->open_connection (provider, cnc, params, username, password);
-	if (retcode) {
-		provider->priv->connections = g_list_append (
-			provider->priv->connections, cnc);
-	}
-	else {
-		/* unref the object if we've got no connections */
-		if (!provider->priv->connections)
-			g_object_unref (G_OBJECT (provider));
-	}
 
 	return retcode;
 }
@@ -281,10 +256,6 @@ gda_server_provider_close_connection (GdaServerProvider *provider, GdaConnection
 	else
 		retcode = TRUE;
 
-	provider->priv->connections = g_list_remove (provider->priv->connections, cnc);
-	if (!provider->priv->connections)
-		g_object_unref (G_OBJECT (provider));
-
 	return retcode;
 }
 
@@ -465,7 +436,7 @@ gda_server_provider_create_operation (GdaServerProvider *provider, GdaConnection
 				      GdaServerOperationType type, 
 				      GdaParameterList *options, GError **error)
 {
-	OpReq **op_req_table = NULL;
+	static OpReq **op_req_table = NULL;
 
 	if (! op_req_table) {
 		op_req_table = g_new0 (OpReq *, GDA_SERVER_OPERATION_NB);
@@ -664,13 +635,69 @@ gda_server_provider_execute_command (GdaServerProvider *provider,
 		GdaServerProviderInfo *info;
 
 		info = gda_server_provider_get_info (provider, cnc);
-		g_print ("==> %s (Provider %s on cnx %p)\n", cmd->text, info->provider_name, cnc);
+		g_print ("COMMAND> %s (Provider %s on cnx %p)\n", cmd->text, info->provider_name, cnc);
 	}
 #endif
 
 	return CLASS (provider)->execute_command (provider, cnc, cmd, params);
 }
 
+/**
+ * gda_server_provider_execute_query
+ * @provider: a #GdaServerProvider object
+ * @cnc: a #GdaConnection object using 
+ * @query: a #GdaQuery 
+ * @params: a #GdaParameterList object obtained using gda_query_get_parameter_list(@query)
+ *
+ * Executes the @query query, and returns:
+ * <itemizedlist>
+ *   <listitem><para>A #GdaDataModel if the query was a SELECT statement and the statement was successufully executed</para></listitem>
+ *   <listitem><para>A #GdaParameterList if the query was not a SELECT 
+ *        query and the query was successufully executed. In this case
+ *        (if the provider supports it), then the #GdaParameterList may contain:
+ *        <itemizedlist>
+ *          <listitem><para>a (gint) #GdaParameter named "IMPACTED_ROWS"</para></listitem>
+ *          <listitem><para>a (GObject) #GdaParameter named "EVENT" which contains a GdaConnectionEvent</para></listitem>
+ *        </itemizedlist>
+ *   </para></listitem>
+ *   <listitem><para>%NULL is in the list if the query could not be executed</para></listitem>
+ * </itemizedlist>
+ *
+ * The differences between this function and gda_server_provider_execute_command() are:
+ * <itemizedlist>
+ *   <listitem><para>Only one query can be executed at a time</para></listitem>
+ *   <listitem><para>If the database provider supports it, prepared statements can be used</para></listitem>
+ *   <listitem><para>Blobs are handled correctly (because blobs are usually not handled using SQL)</para></listitem>
+ * </itemizedlist>
+ *
+ * Returns: a new (#GdaDataModel or #GdaParameterList) object, or %NULL
+ */
+GdaObject *
+gda_server_provider_execute_query (GdaServerProvider *provider,
+				   GdaConnection *cnc,
+				   GdaQuery *query,
+				   GdaParameterList *params)
+{
+	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), NULL);
+	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+	g_return_val_if_fail (CLASS (provider)->execute_query != NULL, NULL);
+
+#ifdef GDA_DEBUG
+	{
+		GdaServerProviderInfo *info;
+		gchar *sql;
+
+		info = gda_server_provider_get_info (provider, cnc);
+		sql = gda_renderer_render_as_sql (GDA_RENDERER (query), params, NULL, 0, NULL);
+		g_print ("QUERY> %s (Provider %s on cnx %p)\n", sql, info->provider_name, cnc);
+		g_free (sql);
+	}
+#endif
+	
+	return CLASS (provider)->execute_query (provider, cnc, query, params);
+}
+
+
 /**
  * gda_server_provider_get_last_insert_id
  * @provider: a #GdaServerProvider object.
@@ -681,6 +708,10 @@ gda_server_provider_execute_command (GdaServerProvider *provider,
  * A connection must be specified, and, optionally, a result set. If not NULL,
  * the provider should try to get the last insert ID for the given result set.
  *
+ * Beware however that the interpretation and usage of this value depends on the
+ * DBMS type being used, see the <link linkend="limitations">limitations</link> 
+ * of each DBMS for more information.
+ *
  * Returns: a string representing the ID of the last inserted row, or NULL
  * if an error occurred or no row has been inserted. It is the caller's
  * reponsibility to free the returned string.
@@ -859,11 +890,6 @@ gda_server_provider_supports_feature (GdaServerProvider *provider,
 			if (!CLASS (provider)->delete_savepoint)
 				retval = FALSE;
 			break;
-		case GDA_CONNECTION_FEATURE_BLOBS:
-			if (!CLASS (provider)->create_blob ||
-			    !CLASS (provider)->fetch_blob)
-				retval = FALSE;
-			break;
 		default:
 			break;
 		}
@@ -882,7 +908,8 @@ gda_server_provider_supports_feature (GdaServerProvider *provider,
  *
  * Get a #GdaDataModel containing the requested information. See <link linkend="libgda-provider-get-schema">this section</link> for more 
  * information on the columns of the returned #GdaDataModel depending on requested @schema, and for the possible
- * parameters of @params.
+ * parameters of @params, see the
+ * <link linkend="libgda-provider-get-schema">get_schema() virtual method for providers</link> for more details.
  *
  * Returns: a new #GdaDataModel, or %NULL if an error occurred.
  */
@@ -908,49 +935,6 @@ gda_server_provider_get_schema (GdaServerProvider *provider,
 	return model;
 }
 
-
-/**
- * gda_server_provider_create_blob
- * @provider: a server provider.
- * @cnc: a #GdaConnection object.
- *
- * Creates a BLOB (Binary Large OBject) with read/write access.
- *
- * Returns: a new #GdaBlob object, or %NULL if the database (or the libgda's provider)
- * does not support BLOBS
- */
-GdaBlob *
-gda_server_provider_create_blob (GdaServerProvider *provider,
-				 GdaConnection *cnc)
-{
-	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-
-	return CLASS (provider)->create_blob (provider, cnc);
-}
-
-/**
- * gda_server_provider_fetch_blob_by_id
- * @provider: a server provider.
- * @cnc: a #GdaConnection object.
- * @sql_id: the SQL ID of the blob to fetch
- *
- * Fetch an existing BLOB (Binary Large OBject) using its SQL ID.
- *
- * Returns: a new #GdaBlob object, or %NULL if the database (or the libgda's provider)
- * does not support BLOBS
- */
-GdaBlob *
-gda_server_provider_fetch_blob_by_id (GdaServerProvider *provider,
-				      GdaConnection *cnc, const gchar *sql_id)
-{
-	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), FALSE);
-	g_return_val_if_fail (GDA_IS_CONNECTION (cnc), FALSE);
-	g_return_val_if_fail (sql_id, FALSE);
-
-	return CLASS (provider)->fetch_blob (provider, cnc, sql_id);
-}
-
 /**
  * gda_server_provider_get_data_handler_gtype
  * @provider: a server provider.
@@ -1009,7 +993,7 @@ gda_server_provider_get_data_handler_dbms (GdaServerProvider *provider,
  *
  * Use @provider to create a new #GValue from a single string representation. 
  *
- * The @prefered_type can optionnaly ask @provider to return a #GValue of the requested type 
+ * The @prefered_type can optionally ask @provider to return a #GValue of the requested type 
  * (but if such a value can't be created from @string, then %NULL is returned); 
  * pass G_TYPE_INVALID if any returned type is acceptable.
  *
@@ -1141,6 +1125,61 @@ gda_server_provider_value_to_sql_string (GdaServerProvider *provider,
 		return NULL;
 }
 
+/**
+ * gda_server_provider_escape_string
+ * @provider: a server provider.
+ * @cnc: a #GdaConnection object, or %NULL
+ * @str: a string to escape
+ *
+ * Escapes @str for use within an SQL command (to avoid SQL injection attacks). Note that the returned value still needs
+ * to be enclosed in single quotes before being used in an SQL statement.
+ *
+ * Returns: a new string suitable to use in SQL statements
+ */
+gchar *
+gda_server_provider_escape_string (GdaServerProvider *provider, GdaConnection *cnc, const gchar *str)
+{
+	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), NULL);
+	if (cnc)
+		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+
+	if (CLASS (provider)->escape_string) {
+		if (! CLASS (provider)->unescape_string)
+			g_warning (_("GdaServerProvider object implements the escape_string() virtual method but "
+				     "does not implement the unescape_string() one, please report this bug."));
+		return (CLASS (provider)->escape_string)(provider, cnc, str);
+	}
+	else 
+		return gda_default_escape_string (str);
+}
+
+/**
+ * gda_server_provider_unescape_string
+ * @provider: a server provider.
+ * @cnc: a #GdaConnection object, or %NULL
+ * @str: a string to escape
+ *
+ * Unescapes @str for use within an SQL command. This is the exact opposite of gda_server_provider_escape_string().
+ *
+ * Returns: a new string
+ */
+gchar *
+gda_server_provider_unescape_string (GdaServerProvider *provider, GdaConnection *cnc, const gchar *str)
+{
+	g_return_val_if_fail (GDA_IS_SERVER_PROVIDER (provider), NULL);
+	if (cnc)
+		g_return_val_if_fail (GDA_IS_CONNECTION (cnc), NULL);
+
+	if (CLASS (provider)->unescape_string) {
+		if (! CLASS (provider)->escape_string)
+			g_warning (_("GdaServerProvider object implements the unescape_string() virtual method but "
+				     "does not implement the escape_string() one, please report this bug."));
+		return (CLASS (provider)->unescape_string)(provider, cnc, str);
+	}
+	else
+		return gda_default_unescape_string (str);
+}
+
 /**
  * gda_server_provider_get_default_dbms_type
  * @provider: a server provider.
