@@ -85,7 +85,7 @@ gda_g_type_from_string (const gchar *str)
  * @string: string to escape
  *
  * Escapes @string to make it understandable by a DBMS. The escape method is very common and replaces any
- * occurence of "'" with "\'" and "\" with "\\".
+ * occurence of "'" with "''" and "\" with "\\"
  */
 gchar *
 gda_default_escape_string (const gchar *string)
@@ -111,7 +111,12 @@ gda_default_escape_string (const gchar *string)
 	ret = g_new0 (gchar, size);
 	retptr = ret;
 	while (*ptr) {
-		if ((*ptr == '\'') || (*ptr == '\\')) {
+		if (*ptr == '\'') {
+			*retptr = '\'';
+			*(retptr+1) = *ptr;
+			retptr += 2;
+		}
+		else if (*ptr == '\\') {
 			*retptr = '\\';
 			*(retptr+1) = *ptr;
 			retptr += 2;
@@ -281,7 +286,7 @@ gda_file_save (const gchar *filename, const gchar *buffer, gint len)
  * gda_utility_table_field_attrs_stringify
  */
 gchar *
-gda_utility_table_field_attrs_stringify (guint attributes)
+gda_utility_table_field_attrs_stringify (GdaValueAttribute attributes)
 {
 	if (attributes & FIELD_AUTO_INCREMENT)
 		return g_strdup ("AUTO_INCREMENT");
@@ -652,9 +657,9 @@ gda_utility_check_data_model (GdaDataModel *model, gint nbcols, ...)
  */
 void
 gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent, 
-				     const gint *cols, gint nb_cols, 
-				     const gint *rows, gint nb_rows,
-				     gboolean use_col_ids)
+					 const gint *cols, gint nb_cols, 
+					 const gint *rows, gint nb_rows,
+					 gboolean use_col_ids)
 {
 	gint nrows, i;
 	gint *rcols, rnb_cols;
@@ -681,10 +686,13 @@ gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent,
 		
 			column = gda_data_model_describe_column (model, rcols [c]);
 			g_object_get (G_OBJECT (column), "id", &id, NULL);
+
 			if (id && *id)
 				col_ids [c] = g_strdup (id);
 			else
 				col_ids [c] = g_strdup_printf ("_%d", c);
+
+			g_free(id);
 		}
 	}
 
@@ -702,24 +710,35 @@ gda_utility_data_model_dump_data_to_xml (GdaDataModel *model, xmlNodePtr parent,
 			row = xmlNewChild (data, NULL,  (xmlChar*)"gda_array_row", NULL);
 			for (c = 0; c < rnb_cols; c++) {
 				GValue *value;
-				gchar *str;
+				gchar *str = NULL;
+				gboolean isnull = FALSE;
 
 				value = (GValue *) gda_data_model_get_value_at (model, rcols [c], rows ? rows [r] : r);
-				if (!value || gda_value_is_null ((GValue *) value))
-					str = NULL;
+				if (!value || gda_value_is_null ((GValue *) value)) 
+					isnull = TRUE;
 				else {
 					if (G_VALUE_TYPE (value) == G_TYPE_BOOLEAN)
 						str = g_strdup (g_value_get_boolean (value) ? "TRUE" : "FALSE");
+					else if (G_VALUE_TYPE (value) == G_TYPE_STRING) {
+						if (!g_value_get_string (value))
+							isnull = TRUE;
+						else
+							str = gda_value_stringify (value);	
+					}
 					else
 						str = gda_value_stringify (value);
 				}
-				if (!use_col_ids) 
-					field = xmlNewChild (row, NULL,  (xmlChar*)"gda_value", (xmlChar*)str);
+				if (!use_col_ids) {
+					if (! str || (str && (*str == 0)))
+						field = xmlNewChild (row, NULL,  (xmlChar*)"gda_value", NULL);
+					else
+						field = xmlNewChild (row, NULL,  (xmlChar*)"gda_value", (xmlChar*)str);
+				}
 				else {
 					field = xmlNewChild (row, NULL,  (xmlChar*)"gda_array_value", (xmlChar*)str);
 					xmlSetProp(field, (xmlChar*)"colid",  (xmlChar*)col_ids [c]);
 				}
-				if (!str)
+				if (isnull)
 					xmlSetProp(field,  (xmlChar*)"isnull", (xmlChar*)"t");
 
 				g_free (str);
@@ -866,9 +885,10 @@ gda_utility_parameter_load_attributes (GdaParameter *param, xmlNodePtr node, GSL
 			
 			if (!isnull) {
 				GValue *value;
+				gchar* nodeval = (gchar*)xmlNodeGetContent (vnode);
 
 				value = g_new0 (GValue, 1);
-				if (! gda_value_set_from_string (value, (gchar*)xmlNodeGetContent (vnode), gdatype)) {
+				if (! gda_value_set_from_string (value, nodeval, gdatype)) {
 					/* error */
 					g_free (value);
 				}
@@ -876,6 +896,8 @@ gda_utility_parameter_load_attributes (GdaParameter *param, xmlNodePtr node, GSL
 					gda_parameter_set_value (param, value);
 					gda_value_free (value);
 				}
+
+				xmlFree(nodeval);
 			}
 			else {
 				gda_parameter_set_value (param, NULL);
