@@ -1,5 +1,5 @@
 /* GDA LDAP Provider
- * Copyright (C) 1998 - 2005 The GNOME Foundation
+ * Copyright (C) 1998 - 2007 The GNOME Foundation
  *
  * AUTHORS:
  * 	   Michael Lausch <michael@lausch.at>
@@ -25,12 +25,21 @@
 #include <glib/gi18n-lib.h>
 #include <libgda/gda-config.h>
 #include "gda-ldap-provider.h"
+#include <libgda/gda-server-provider-extra.h>
 
+static gchar      *module_path = NULL;
 const gchar       *plugin_get_name (void);
 const gchar       *plugin_get_description (void);
 gchar             *plugin_get_dsn_spec (void);
 GdaServerProvider *plugin_create_provider (void);
 
+void
+plugin_init (const gchar *real_path)
+{
+	if (real_path)
+		module_path = g_strdup (real_path);
+}
+
 const gchar *
 plugin_get_name (void)
 {
@@ -46,18 +55,15 @@ plugin_get_description (void)
 gchar *
 plugin_get_dsn_spec (void)
 {
-	gchar *specs, *file;
-	gsize len;
-
-	file = g_build_filename (LIBGDA_DATA_DIR, "ldap_specs_dsn.xml", NULL);
-	if (g_file_get_contents (file, &specs, &len, NULL)) 
-		return specs;
-	else
-		return NULL;
+	return gda_server_provider_load_file_contents (module_path, LIBGDA_DATA_DIR, "ldap_specs_dsn.xml");
 }
 
 GdaServerProvider *
 plugin_create_provider (void)
 {
-	return gda_ldap_provider_new ();
+	GdaServerProvider *prov;
+
+	prov = gda_ldap_provider_new ();
+	g_object_set_data (prov, "GDA_PROVIDER_DIR", module_path);
+	return prov;
 }
