@@ -1,5 +1,5 @@
 /* GDA MySQL provider
- * Copyright (C) 1998 - 2005 The GNOME Foundation.
+ * Copyright (C) 1998 - 2006 The GNOME Foundation.
  *
  * AUTHORS:
  *      Michael Lausch <michael@lausch.at>
@@ -23,7 +23,7 @@
  * Boston, MA 02111-1307, USA.
  */
 
-#include <libgda/gda-intl.h>
+#include <glib/gi18n-lib.h>
 #include <stdlib.h>
 #include <string.h>
 #include "gda-mysql.h"
@@ -33,12 +33,12 @@
 #define PARENT_TYPE GDA_TYPE_DATA_MODEL_ARRAY
 
 struct _GdaMysqlRecordsetPrivate {
-	MYSQL_RES *mysql_res;
-	gint mysql_res_rows;
+	MYSQL_RES     *mysql_res;
+	gint           mysql_res_rows;
 	GdaConnection *cnc;
-	gint ncolumns;
-	gchar *table_name;
-	gboolean row_sync;
+	gint           ncolumns;
+	gchar         *table_name;
+	gboolean       row_sync;
 };
 
 static void gda_mysql_recordset_class_init (GdaMysqlRecordsetClass *klass);
@@ -46,14 +46,14 @@ static void gda_mysql_recordset_init       (GdaMysqlRecordset *recset,
 					    GdaMysqlRecordsetClass *klass);
 static void gda_mysql_recordset_finalize   (GObject *object);
 
-static gint gda_mysql_recordset_get_n_rows			     (GdaDataModelBase *model);
-static GdaColumn *gda_mysql_recordset_describe_column (GdaDataModelBase *model, gint col);
-static const GdaRow *gda_mysql_recordset_get_row		     (GdaDataModelBase *model, gint row);
-static const GdaValue *gda_mysql_recordset_get_value_at		     (GdaDataModelBase *model, gint col, gint row);
-static gboolean gda_mysql_recordset_is_updatable		     (GdaDataModelBase *model);
-static gboolean gda_mysql_recordset_append_row			     (GdaDataModelBase *model, GdaRow *row);
-static gboolean gda_mysql_recordset_remove_row			     (GdaDataModelBase *model, const GdaRow *row);
-static gboolean gda_mysql_recordset_update_row			     (GdaDataModelBase *model, const GdaRow *row);
+static gint gda_mysql_recordset_get_n_rows			     (GdaDataModelRow *model);
+static GdaRow *gda_mysql_recordset_get_row		     (GdaDataModelRow *model, 
+							      gint row, GError **error);
+static const GValue *gda_mysql_recordset_get_value_at		     (GdaDataModelRow *model, gint col, gint row);
+static gboolean gda_mysql_recordset_is_updatable		     (GdaDataModelRow *model);
+static gboolean gda_mysql_recordset_append_row			     (GdaDataModelRow *model, GdaRow *row, GError **error);
+static gboolean gda_mysql_recordset_remove_row			     (GdaDataModelRow *model, GdaRow *row, GError **error);
+static gboolean gda_mysql_recordset_update_row			     (GdaDataModelRow *model, GdaRow *row, GError **error);
 
 static GObjectClass *parent_class = NULL;
 
@@ -68,7 +68,7 @@ static void
 gda_mysql_recordset_class_init (GdaMysqlRecordsetClass *klass)
 {
 	GObjectClass *object_class = G_OBJECT_CLASS (klass);
-	GdaDataModelBaseClass *model_class = GDA_DATA_MODEL_BASE_CLASS (klass);
+	GdaDataModelRowClass *model_class = GDA_DATA_MODEL_ROW_CLASS (klass);
 
 	parent_class = g_type_class_peek_parent (klass);
 
@@ -112,7 +112,7 @@ gda_mysql_recordset_finalize (GObject *object)
 }
 
 static void
-fill_gda_value (GdaValue *gda_value, enum enum_field_types type, gchar *value,
+fill_gda_value (GValue *gda_value, enum enum_field_types type, gchar *value,
 		unsigned long length, gboolean is_unsigned)
 {
 	if (!value) {
@@ -123,39 +123,52 @@ fill_gda_value (GdaValue *gda_value, enum enum_field_types type, gchar *value,
 	switch (type) {
 	case FIELD_TYPE_DECIMAL :
 	case FIELD_TYPE_DOUBLE :
-		gda_value_set_double (gda_value, atof (value));
+#if NDB_VERSION_MAJOR >= 5
+	case FIELD_TYPE_NEWDECIMAL :
+#endif
+		gda_value_reset_with_type (gda_value, G_TYPE_DOUBLE);
+		g_value_set_double (gda_value, atof (value));
 		break;
 	case FIELD_TYPE_FLOAT :
-		gda_value_set_single (gda_value, atof (value));
+		gda_value_reset_with_type (gda_value, G_TYPE_FLOAT);
+		g_value_set_float (gda_value, atof (value));
 		break;
 	case FIELD_TYPE_LONG :
 		if (is_unsigned) {
-			gda_value_set_uinteger (gda_value, strtoul (value, NULL, 0));
+			gda_value_reset_with_type (gda_value, G_TYPE_UINT);
+			g_value_set_uint (gda_value, strtoul (value, NULL, 0));
 			break;
 		}
 	case FIELD_TYPE_YEAR :
-		gda_value_set_integer (gda_value, atol (value));
+		gda_value_reset_with_type (gda_value, G_TYPE_INT);
+		g_value_set_int (gda_value, atol (value));
 		break;
 	case FIELD_TYPE_LONGLONG :
 	case FIELD_TYPE_INT24 :
 		if (is_unsigned) {
-			gda_value_set_biguint (gda_value, strtoull (value, NULL, 0));
+			gda_value_reset_with_type (gda_value, G_TYPE_UINT64);
+			g_value_set_uint64 (gda_value, strtoull (value, NULL, 0));
 		} else {
-			gda_value_set_bigint (gda_value, atoll (value));
+			gda_value_reset_with_type (gda_value, G_TYPE_INT64);
+			g_value_set_int64 (gda_value, atoll (value));
 		}
 		break;
 	case FIELD_TYPE_SHORT :
 		if (is_unsigned) {
-			gda_value_set_smalluint (gda_value, atoi (value));
+			gda_value_reset_with_type (gda_value, GDA_TYPE_USHORT);
+			gda_value_set_ushort (gda_value, atoi (value));
 		} else {
-			gda_value_set_smallint (gda_value, atoi (value));
+			gda_value_reset_with_type (gda_value, GDA_TYPE_SHORT);
+			gda_value_set_short (gda_value, atoi (value));
 		}
 		break;
 	case FIELD_TYPE_TINY :
 		if (is_unsigned) {
-			gda_value_set_tinyuint (gda_value, atoi (value));
+			gda_value_reset_with_type (gda_value, G_TYPE_UCHAR);
+			g_value_set_uchar (gda_value, atoi (value));
 		} else {
-			gda_value_set_tinyint (gda_value, atoi (value));
+			gda_value_reset_with_type (gda_value, G_TYPE_CHAR);
+			g_value_set_char (gda_value, atoi (value));
 		}
 		break;
 	case FIELD_TYPE_TINY_BLOB :
@@ -165,6 +178,7 @@ fill_gda_value (GdaValue *gda_value, enum enum_field_types type, gchar *value,
 		GdaBinary bin;
 		bin.data = value;
 		bin.binary_length = length;
+		gda_value_reset_with_type (gda_value, GDA_TYPE_BINARY);
 		gda_value_set_binary (gda_value, &bin);
 		break;
 	}
@@ -173,27 +187,31 @@ fill_gda_value (GdaValue *gda_value, enum enum_field_types type, gchar *value,
 		/* FIXME: we might get "[VAR]CHAR(20) BINARY" type with \0 inside
 		   We should check for BINARY flag and treat it like a BLOB
 		 */
-		gda_value_set_string (gda_value, value);
+		gda_value_reset_with_type (gda_value, G_TYPE_STRING);
+		g_value_set_string (gda_value, value);
 		break;
 	case FIELD_TYPE_DATE :
-		gda_value_set_from_string (gda_value, value, GDA_VALUE_TYPE_DATE);
+		gda_value_set_from_string (gda_value, value, G_TYPE_DATE);
 		break;
 	case FIELD_TYPE_TIME :
-		gda_value_set_from_string (gda_value, value, GDA_VALUE_TYPE_TIME);
+		gda_value_set_from_string (gda_value, value, GDA_TYPE_TIME);
 		break;
 	case FIELD_TYPE_TIMESTAMP :
 	case FIELD_TYPE_DATETIME :
-		gda_value_set_from_string (gda_value, value, GDA_VALUE_TYPE_TIMESTAMP);
+		gda_value_set_from_string (gda_value, value, GDA_TYPE_TIMESTAMP);
 		break;
 	case FIELD_TYPE_NULL :
+		gda_value_set_null (gda_value);
+		break;
 	case FIELD_TYPE_NEWDATE :
 	case FIELD_TYPE_ENUM :
 	case FIELD_TYPE_SET : /* FIXME */
-		gda_value_set_string (gda_value, value);
+		gda_value_reset_with_type (gda_value, G_TYPE_STRING);
+		g_value_set_string (gda_value, value);
 		break;
 	default :
 		g_printerr ("Unknown MySQL datatype.  This is fishy, but continuing anyway.\n");
-		gda_value_set_string (gda_value, value);
+		g_value_set_string (gda_value, value);
 	}
 }
 
@@ -209,7 +227,6 @@ fetch_row (GdaMysqlRecordset *recset, gulong rownum)
 	MYSQL_ROW mysql_row;
 	unsigned long *mysql_lengths;
 
-
 	g_return_val_if_fail (GDA_IS_MYSQL_RECORDSET (recset), NULL);
 	g_return_val_if_fail (recset->priv != NULL, 0);
 
@@ -239,7 +256,7 @@ fetch_row (GdaMysqlRecordset *recset, gulong rownum)
 	row = gda_row_new (GDA_DATA_MODEL (recset), field_count);
 
 	for (i = 0; i < field_count; i++) {
-		fill_gda_value ((GdaValue*) gda_row_get_value (row, i),
+		fill_gda_value ((GValue*) gda_row_get_value (row, i),
 				mysql_fields[i].type,
 				mysql_row[i],
 				mysql_lengths[i],
@@ -254,7 +271,7 @@ fetch_row (GdaMysqlRecordset *recset, gulong rownum)
  */
 
 static gint
-gda_mysql_recordset_get_n_rows (GdaDataModelBase *model)
+gda_mysql_recordset_get_n_rows (GdaDataModelRow *model)
 {
 	GdaMysqlRecordset *recset = (GdaMysqlRecordset *) model;
 
@@ -264,24 +281,25 @@ gda_mysql_recordset_get_n_rows (GdaDataModelBase *model)
 	if (recset->priv->row_sync == FALSE)
 		return recset->priv->mysql_res_rows;
 	else
-		return GDA_DATA_MODEL_BASE_CLASS (parent_class)->get_n_rows (model);
+		return GDA_DATA_MODEL_ROW_CLASS (parent_class)->get_n_rows (model);
 }
 
-static const GdaRow *
-gda_mysql_recordset_get_row (GdaDataModelBase *model, gint row)
+static GdaRow *
+gda_mysql_recordset_get_row (GdaDataModelRow *model, gint row, GError **error)
 {
 	GdaMysqlRecordset *recset = (GdaMysqlRecordset *) model;
 	GdaMysqlRecordsetPrivate *priv_data;
 	gint fetched_rows;
 	gint i;
-	GdaRow *row_list, *fields = NULL;
+	GdaRow *row_list;
 
 	g_return_val_if_fail (GDA_IS_MYSQL_RECORDSET (recset), NULL);
 	g_return_val_if_fail (recset->priv != NULL, 0);
 
-	row_list = (GdaRow *) GDA_DATA_MODEL_BASE_CLASS (parent_class)->get_row (model, row);
+	row_list = (GdaRow *) GDA_DATA_MODEL_ROW_CLASS (parent_class)->get_row (model, row, 
+										error);
 	if (row_list != NULL)
-		return (const GdaRow *)row_list;	
+		return row_list;	
 
 	priv_data = recset->priv;
 	if (!priv_data->mysql_res) {
@@ -291,12 +309,18 @@ gda_mysql_recordset_get_row (GdaDataModelBase *model, gint row)
 	}
 	
 	if (row < 0 || row > priv_data->mysql_res_rows) {
-		gda_connection_add_event_string (priv_data->cnc,
-						_("Row number out of range"));
+		gchar *str;
+
+		str = g_strdup_printf (_("Row number out of range 0 - %d"),
+				       priv_data->mysql_res_rows - 1);
+		gda_connection_add_event_string (priv_data->cnc, str);
+		g_set_error (error, 0, 0, str);
+		g_free (str);
+						 
 		return NULL;
 	}
 
-	fetched_rows = GDA_DATA_MODEL_BASE_CLASS (parent_class)->get_n_rows (model);
+	fetched_rows = GDA_DATA_MODEL_ROW_CLASS (parent_class)->get_n_rows (model);
 
 	gda_data_model_freeze (GDA_DATA_MODEL (recset));
 
@@ -305,30 +329,31 @@ gda_mysql_recordset_get_row (GdaDataModelBase *model, gint row)
 		if (!row_list)
 			return NULL;
 
-		if (GDA_DATA_MODEL_BASE_CLASS (parent_class)->append_row (model, row_list) == FALSE)
+		if (! GDA_DATA_MODEL_ROW_CLASS (parent_class)->append_row (model, row_list, NULL))
 			return NULL;
 	}
 
 	gda_data_model_thaw (GDA_DATA_MODEL (recset));
 
-
-	return (const GdaRow *) row_list;
+	return row_list;
 }
 
-static const GdaValue *
-gda_mysql_recordset_get_value_at (GdaDataModelBase *model, gint col, gint row)
+static const GValue *
+gda_mysql_recordset_get_value_at (GdaDataModelRow *model, gint col, gint row)
 {
 	GdaMysqlRecordset *recset = (GdaMysqlRecordset *) model;
 	GdaMysqlRecordsetPrivate *priv_data;
-	const GdaValue *value;
+	const GValue *value;
 	const GdaRow *fields;
 
 	g_return_val_if_fail (GDA_IS_MYSQL_RECORDSET (recset), NULL);
 	g_return_val_if_fail (recset->priv != NULL, 0);
 
-	value = GDA_DATA_MODEL_BASE_CLASS (parent_class)->get_value_at (model, col, row);
-	if (value != NULL)
-		return value;
+	if (row < GDA_DATA_MODEL_ROW_CLASS (parent_class)->get_n_rows (model)) {
+		value = GDA_DATA_MODEL_ROW_CLASS (parent_class)->get_value_at (model, col, row);
+		if (value != NULL)
+			return value;
+	}
 
 	priv_data = recset->priv;
         if (!priv_data->mysql_res) {
@@ -349,24 +374,24 @@ gda_mysql_recordset_get_value_at (GdaDataModelBase *model, gint col, gint row)
 		return NULL;
 	}	
 
-	fields = gda_mysql_recordset_get_row (model, row);
+	fields = gda_mysql_recordset_get_row (model, row, NULL);
 	return fields != NULL ? gda_row_get_value ((GdaRow *) fields, col) : NULL;
 }
 
 static gboolean
-gda_mysql_recordset_is_updatable (GdaDataModelBase *model)
+gda_mysql_recordset_is_updatable (GdaDataModelRow *model)
 {
 	GdaCommandType cmd_type;
 	GdaMysqlRecordset *recset = (GdaMysqlRecordset *) model;
 
 	g_return_val_if_fail (GDA_IS_MYSQL_RECORDSET (recset), FALSE);
 	
-	cmd_type = gda_data_model_get_command_type (GDA_DATA_MODEL (model));
+	g_object_get (G_OBJECT (model), "command_type", &cmd_type, NULL);
 	return cmd_type == GDA_COMMAND_TYPE_TABLE ? TRUE : FALSE;
 }
 
 static gboolean
-gda_mysql_recordset_append_row (GdaDataModelBase *model, GdaRow *row)
+gda_mysql_recordset_append_row (GdaDataModelRow *model, GdaRow *row, GError **error)
 {
 	GdaMysqlRecordset *recset = (GdaMysqlRecordset *) model;
 	GdaMysqlRecordsetPrivate *priv_data;
@@ -410,7 +435,7 @@ gda_mysql_recordset_append_row (GdaDataModelBase *model, GdaRow *row)
 	/* check if all results are loaded in array, if not do so */
 	if (priv_data->row_sync == FALSE)
 	{
-		fetched_rows = GDA_DATA_MODEL_BASE_CLASS (parent_class)->get_n_rows (model);
+		fetched_rows = GDA_DATA_MODEL_ROW_CLASS (parent_class)->get_n_rows (model);
 
 		gda_data_model_freeze (GDA_DATA_MODEL (recset));
 
@@ -422,7 +447,7 @@ gda_mysql_recordset_append_row (GdaDataModelBase *model, GdaRow *row)
 				return FALSE;
 			}
 
-			if (GDA_DATA_MODEL_BASE_CLASS (parent_class)->append_row (model, row_list) == FALSE) {
+			if (!GDA_DATA_MODEL_ROW_CLASS (parent_class)->append_row (model, row_list, NULL)) {
 				gda_connection_add_event_string (priv_data->cnc,
 						_("Can not synchronize array with MySQL result set"));
 				return FALSE;
@@ -483,7 +508,7 @@ gda_mysql_recordset_append_row (GdaDataModelBase *model, GdaRow *row)
 	g_string_free (sql_value, TRUE);
 
 	/* append row in the array */
-	if (GDA_DATA_MODEL_BASE_CLASS (parent_class)->append_row (model, row) == FALSE) {
+	if (! GDA_DATA_MODEL_ROW_CLASS (parent_class)->append_row (model, row, NULL)) {
 		gda_connection_add_event_string (priv_data->cnc,
 			_("Can not append row to data model"));
 		return FALSE;
@@ -493,7 +518,7 @@ gda_mysql_recordset_append_row (GdaDataModelBase *model, GdaRow *row)
 }
 
 static gboolean
-gda_mysql_recordset_remove_row (GdaDataModelBase *model, const GdaRow *row)
+gda_mysql_recordset_remove_row (GdaDataModelRow *model, GdaRow *row, GError **error)
 {
 	GdaMysqlRecordset *recset = (GdaMysqlRecordset *) model;
 	GdaMysqlRecordsetPrivate *priv_data;
@@ -523,6 +548,8 @@ gda_mysql_recordset_remove_row (GdaDataModelBase *model, const GdaRow *row)
 	
 	/* checks if the given row belongs to the given model */
 	if (gda_row_get_model ((GdaRow *) row) != GDA_DATA_MODEL (model)) {
+		g_set_error (error, 0, 0,
+			     _("Given row doesn't belong to the model."));
 		gda_connection_add_event_string (priv_data->cnc,
 						_("Given row doesn't belong to the model."));
 		return FALSE;
@@ -530,6 +557,8 @@ gda_mysql_recordset_remove_row (GdaDataModelBase *model, const GdaRow *row)
 
 	/* checks if the table name has been guessed */
 	if (priv_data->table_name == NULL) {
+		g_set_error (error, 0, 0,
+			     _("Table name could not be guessed"));
 		gda_connection_add_event_string (priv_data->cnc,
 						_("Table name could not be guessed"));
 		return FALSE;
@@ -538,19 +567,21 @@ gda_mysql_recordset_remove_row (GdaDataModelBase *model, const GdaRow *row)
 	/* check if all results are loaded in array, if not do so */
 	if (priv_data->row_sync == FALSE)
 	{
-		fetched_rows = GDA_DATA_MODEL_BASE_CLASS (parent_class)->get_n_rows (model);
+		fetched_rows = GDA_DATA_MODEL_ROW_CLASS (parent_class)->get_n_rows (model);
 
 		gda_data_model_freeze (GDA_DATA_MODEL (recset));
 
 		for (i = fetched_rows; i < priv_data->mysql_res_rows; i++) {
 			row_list = fetch_row (recset, i);
 			if (!row_list) {
+				g_set_error (error, 0, 0,
+					     _("Can not synchronize array with MySQL result set"));
 				gda_connection_add_event_string (priv_data->cnc,
 						_("Can not synchronize array with MySQL result set"));
 				return FALSE;
 			}
 
-			if (GDA_DATA_MODEL_BASE_CLASS (parent_class)->append_row (model, row_list) == FALSE) {
+			if (! GDA_DATA_MODEL_ROW_CLASS (parent_class)->append_row (model, row_list, error)) {
 				gda_connection_add_event_string (priv_data->cnc,
 						_("Can not synchronize array with MySQL result set"));
 				return FALSE;
@@ -603,6 +634,8 @@ gda_mysql_recordset_remove_row (GdaDataModelBase *model, const GdaRow *row)
 	}
 
 	if (uk == 0) {
+		g_set_error (error, 0, 0,
+			     _("Model doesn't have at least one unique key."));
 		gda_connection_add_event_string (priv_data->cnc,
 						_("Model doesn't have at least one unique key."));
 		g_free (query_where);
@@ -630,7 +663,9 @@ gda_mysql_recordset_remove_row (GdaDataModelBase *model, const GdaRow *row)
 	g_free (query_where);
 
 	/* remove row from the array */
-	if (GDA_DATA_MODEL_BASE_CLASS (parent_class)->remove_row (model, row) == FALSE) {
+	if (! GDA_DATA_MODEL_ROW_CLASS (parent_class)->remove_row (model, row, NULL)) {
+		g_set_error (error, 0, 0,
+			     _("Can not remove row from data model"));
 		gda_connection_add_event_string (priv_data->cnc,
 			_("Can not remove row from data model"));
 		return FALSE;
@@ -640,7 +675,7 @@ gda_mysql_recordset_remove_row (GdaDataModelBase *model, const GdaRow *row)
 }
 
 static gboolean
-gda_mysql_recordset_update_row (GdaDataModelBase *model, const GdaRow *row)
+gda_mysql_recordset_update_row (GdaDataModelRow *model, GdaRow *row, GError **error)
 {
 	GdaMysqlRecordset *recset = (GdaMysqlRecordset *) model;
 	GdaMysqlRecordsetPrivate *priv_data;
@@ -662,6 +697,8 @@ gda_mysql_recordset_update_row (GdaDataModelBase *model, const GdaRow *row)
 
 	/* checks for valid MySQL handle */
         if (!priv_data->mysql_res) {
+		g_set_error (error, 0, 0,
+			     _("Invalid MySQL handle"));
 		gda_connection_add_event_string (priv_data->cnc, _("Invalid MySQL handle"));
 		return FALSE;
 	}
@@ -671,6 +708,8 @@ gda_mysql_recordset_update_row (GdaDataModelBase *model, const GdaRow *row)
 	
 	/* checks if the given row belongs to the given model */
 	if (gda_row_get_model ((GdaRow *) row) != GDA_DATA_MODEL (model)) {
+		g_set_error (error, 0, 0,
+			     _("Given row doesn't belong to the model."));
 		gda_connection_add_event_string (priv_data->cnc,
 						_("Given row doesn't belong to the model."));
 		return FALSE;
@@ -678,6 +717,8 @@ gda_mysql_recordset_update_row (GdaDataModelBase *model, const GdaRow *row)
 
 	/* checks if the table name has been guessed */
 	if (priv_data->table_name == NULL) {
+		g_set_error (error, 0, 0,
+			     _("Table name could not be guessed."));
 		gda_connection_add_event_string (priv_data->cnc,
 						_("Table name could not be guessed."));
 		return FALSE;
@@ -751,6 +792,8 @@ gda_mysql_recordset_update_row (GdaDataModelBase *model, const GdaRow *row)
 	}
 
 	if (uk == 0) {
+		g_set_error (error, 0, 0,
+			     _("Model does not have at least one non-modified unique key."));	     
 		gda_connection_add_event_string (priv_data->cnc,
 						_("Model does not have at least one non-modified unique key."));
 		g_free (query_set);
@@ -760,6 +803,8 @@ gda_mysql_recordset_update_row (GdaDataModelBase *model, const GdaRow *row)
 	}
 
 	if (nuk == 0) {
+		g_set_error (error, 0, 0,
+			     _("Model does not have any non-unique values to update."));
 		gda_connection_add_event_string (priv_data->cnc,
 						_("Model does not have any non-unique values to update."));
 		g_free (query_set);
@@ -782,8 +827,10 @@ gda_mysql_recordset_update_row (GdaDataModelBase *model, const GdaRow *row)
 	/* execute update command */
 	rc = mysql_real_query (mysql, query, strlen (query));
 	if (rc != 0) {
-		gda_connection_add_event (
-			priv_data->cnc, gda_mysql_make_error (mysql));
+		GdaConnectionEvent *event = gda_mysql_make_error (mysql);
+		gda_connection_add_event (priv_data->cnc, event);
+		g_set_error (error, 0, 0,
+			     gda_connection_event_get_description (event));
 		return FALSE;
 	}
 	
@@ -872,7 +919,7 @@ gda_mysql_recordset_new (GdaConnection *cnc, MYSQL_RES *mysql_res, MYSQL *mysql)
 			gda_column_set_defined_size (column, mysql_field->length);
 			gda_column_set_table (column, mysql_field->table);
 			gda_column_set_scale (column, mysql_field->decimals);
-			gda_column_set_gdatype (column, gda_mysql_type_to_gda (mysql_field->type,
+			gda_column_set_g_type (column, gda_mysql_type_to_gda (mysql_field->type,
 									       mysql_field->flags & UNSIGNED_FLAG));
 			gda_column_set_allow_null (column, !IS_NOT_NULL (mysql_field->flags));
 			gda_column_set_primary_key (column, IS_PRI_KEY (mysql_field->flags));
@@ -883,3 +930,4 @@ gda_mysql_recordset_new (GdaConnection *cnc, MYSQL_RES *mysql_res, MYSQL *mysql)
 
 	return model;
 }
+
