@@ -1,8 +1,9 @@
 /* GDA Oracle Provider
- * Copyright (C) 1998-2002 The GNOME Foundation
+ * Copyright (C) 1998 - 2007 The GNOME Foundation
  *
  * AUTHORS:
  * 	Tim Coleman <tim@timcoleman.com>
+ *      Vivien Malerba <malerba@gnome-db.org>
  *
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Library General Public
@@ -19,8 +20,22 @@
  * Software Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  */
 
-#include <libgda/gda-intl.h>
+#include <glib/gi18n-lib.h>
 #include "gda-oracle-provider.h"
+#include <libgda/gda-server-provider-extra.h>
+
+static gchar      *module_path = NULL;
+const gchar       *plugin_get_name (void);
+const gchar       *plugin_get_description (void);
+gchar             *plugin_get_dsn_spec (void);
+GdaServerProvider *plugin_create_provider (void);
+
+void
+plugin_init (const gchar *real_path)
+{
+        if (real_path)
+                module_path = g_strdup (real_path);
+}
 
 const gchar *
 plugin_get_name (void)
@@ -34,21 +49,18 @@ plugin_get_description (void)
 	return _("Provider for Oracle databases");
 }
 
-GList *
-plugin_get_connection_params (void)
+gchar *
+plugin_get_dsn_spec (void)
 {
-	GList *list = NULL;
-
-	list = g_list_append (list,
-			      gda_provider_parameter_info_new_full ("TNSNAME", _("TNS Name"),
-								    _("Name of the TNS configuration to use for connecting"),
-								    GDA_VALUE_TYPE_STRING));
-
-	return list;
+	return gda_server_provider_load_file_contents (module_path, LIBGDA_DATA_DIR, "oracle_specs_dsn.xml");
 }
 
 GdaServerProvider *
 plugin_create_provider (void)
 {
-	return gda_oracle_provider_new ();
+	GdaServerProvider *prov;
+
+        prov = gda_oracle_provider_new ();
+        g_object_set_data (prov, "GDA_PROVIDER_DIR", module_path);
+        return prov;
 }
