@@ -125,6 +125,9 @@ static const gchar* gda_sqlite_provider_get_default_dbms_type (GdaServerProvider
 							       GdaConnection *cnc,
 							       GType type);
 
+static GList *process_sql_commands (GList *reclist, GdaConnection *cnc,
+				    const gchar *sql, GdaCommandOptions options);
+
 static GObjectClass *parent_class = NULL;
 
 typedef struct {
@@ -452,6 +455,17 @@ process_sql_commands (GList *reclist, GdaConnection *cnc,
 		return NULL;
 	}
 
+	/* HACK: force SQLite to reparse the schema and thus discover new tables if necessary */
+	{
+		gint status;
+		sqlite3_stmt *stmt = NULL;
+		status = sqlite3_prepare_v2 (scnc->connection, "SELECT 1 FROM sqlite_master LIMIT 1", -1, &stmt, NULL);
+		if (status == SQLITE_OK)
+			sqlite3_step (stmt);
+		if (stmt)
+			sqlite3_finalize (stmt);
+	}
+
 	/* parse SQL string, which can contain several commands, separated by ';' */
 	arr = sql_split (sql);
 	if (arr) {
@@ -1309,7 +1323,7 @@ get_table_fields (GdaConnection *cnc, GdaParameterList *params)
 		else {
 			value = gda_row_get_value (row, 3);
 			g_value_set_boolean (nvalue = gda_value_new (G_TYPE_BOOLEAN), 
-					     g_value_get_int ((GValue *) value) ? FALSE : TRUE);
+					     g_value_get_int ((GValue *) value) ? TRUE : FALSE);
 		}
 		rowlist = g_list_append (rowlist, nvalue);
 
